@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2022 Christopher Davis
+// Copyright (c) 2021-2023 Christopher Davis
 // Copyright (c) 2022-2023 Sophie Herold
 // Copyright (c) 2022 Maximiliano Sandoval R
 // Copyright (c) 2023 Lubosz Sarnecki
@@ -41,9 +41,8 @@ use adw::{prelude::*, subclass::prelude::*};
 use arc_swap::ArcSwap;
 use futures::prelude::*;
 use once_cell::sync::Lazy;
-use once_cell::unsync::OnceCell;
 
-use std::cell::{Cell, RefCell};
+use std::cell::{Cell, OnceCell, RefCell};
 use std::sync::Arc;
 
 /// Default background color around images and behind transparent images
@@ -342,13 +341,13 @@ mod imp {
             let scroll_controller =
                 gtk::EventControllerScroll::new(gtk::EventControllerScrollFlags::VERTICAL);
 
-            scroll_controller.connect_scroll(glib::clone!(@weak obj => @default-return gtk::Inhibit(false), move |event, _, y| {
+            scroll_controller.connect_scroll(glib::clone!(@weak obj => @default-return glib::Propagation::Proceed, move |event, _, y| {
                 // use Ctrl key as modifier for vertical scrolling
-                if event.current_event_state() == gdk::ModifierType::CONTROL_MASK
-                    || event.current_event_state() == gdk::ModifierType::SHIFT_MASK
+                if event.current_event_state().contains(gdk::ModifierType::CONTROL_MASK)
+                    || event.current_event_state().contains(gdk::ModifierType::SHIFT_MASK)
                 {
                     // propagate event to scrolled window
-                    return gtk::Inhibit(false);
+                    return glib::Propagation::Proceed;
                 }
 
                 // touchpads do zoom via gestures only
@@ -356,7 +355,7 @@ mod imp {
                     == Some(gdk::InputSource::Touchpad)
                 {
                     // propagate event to scrolled window
-                    return gtk::Inhibit(false);
+                    return glib::Propagation::Proceed;
                 }
 
                 // Use exponential scaling since zoom is always multiplicative with the existing value
@@ -380,7 +379,7 @@ mod imp {
                     }
 
                 // do not propagate event to scrolled window
-                gtk::Inhibit(true)
+                glib::Propagation::Stop
             }));
 
             obj.add_controller(scroll_controller);
@@ -422,14 +421,16 @@ mod imp {
 
             drag_gesture.connect_drag_begin(glib::clone!(@weak obj => move |gesture, _, _| {
                 // Allow only left and middle button
-                if ![1, 2].contains(&gesture.current_button()) {
+                if ![1, 2].contains(&gesture.current_button())
+                    // Drag gesture for touchscreens is handled by ScrolledWindow
+                    || gesture.device().map(|x| x.source()) == Some(gdk::InputSource::Touchscreen)
+                {
                     gesture.set_state(gtk::EventSequenceState::Denied);
                     return;
                 }
 
                 if obj.is_hscrollable() || obj.is_vscrollable() {
                     obj.cancel_deceleration();
-                    gesture.set_state(gtk::EventSequenceState::Claimed);
                     obj.set_cursor(gdk::Cursor::from_name("grabbing", None).as_ref());
                     obj.imp().last_drag_value.set(Some((0., 0.)));
                 } else {
@@ -591,8 +592,12 @@ mod imp {
             self.widget_dimensions.set((width, height));
             obj.configure_adjustments();
 
-            // Get potentially missing tiles for enlarged viewing area
-            obj.request_tiles();
+            // Avoid updates for first size_allocate with zoom not set yet
+            if obj.is_loaded() {
+                // Get potentially missing tiles for enlarged viewing area
+
+                obj.request_tiles();
+            }
         }
 
         // called when the widget content should be re-rendered
@@ -608,7 +613,7 @@ mod imp {
 
             let applicable_zoom = widget.applicable_zoom();
 
-            let scaling_filter = if self.format.borrow().map_or(false, |x| x.is_svg()) {
+            let scaling_filter = if self.format.borrow().as_ref().map_or(false, |x| x.is_svg()) {
                 // Looks better in SVG animations and avoids rendering issues
                 gsk::ScalingFilter::Linear
             } else if applicable_zoom < 1. {
@@ -770,7 +775,7 @@ impl LpImage {
         // Reset background color for reloads
         self.set_background_color(None);
 
-        let (decoder_res, mut decoder_update) = Decoder::new(file.clone(), tiles.clone()).await;
+        let (decoder, mut decoder_update) = Decoder::new(file.clone(), tiles.clone()).await;
 
         let weak_obj = self.downgrade();
         spawn(async move {
@@ -782,10 +787,7 @@ impl LpImage {
             log::debug!("Stopped listening to decoder since sender is gone");
         });
 
-        if let Ok(decoder) = decoder_res {
-            let decoder = Arc::new(decoder);
-            self.imp().decoder.replace(Some(decoder));
-        }
+        self.imp().decoder.replace(Some(Arc::new(decoder)));
     }
 
     /// Called when decoder sends update
@@ -816,7 +818,6 @@ impl LpImage {
                 self.queue_draw();
                 imp.frame_buffer.rcu(|tiles| {
                     let mut new_tiles = (**tiles).clone();
-                    // TODO: Use an area larger than the viewport
                     new_tiles.cleanup(imp.zoom_target.get(), self.preload_area());
                     new_tiles
                 });
@@ -836,13 +837,11 @@ impl LpImage {
             DecoderUpdate::Format(format) => {
                 imp.format.replace(Some(format));
                 self.notify("format-name");
-
-                // Do the animation part for animated formats
-                if format.is_animated() {
-                    let callback_id = self
-                        .add_tick_callback(glib::clone!(@weak self as obj => @default-return glib::Continue(false), move |_, clock| obj.tick_callback(clock)));
-                    imp.tick_callback.replace(Some(callback_id));
-                }
+            }
+            DecoderUpdate::Animated => {
+                let callback_id = self
+                        .add_tick_callback(glib::clone!(@weak self as obj => @default-return glib::ControlFlow::Break, move |_, clock| obj.tick_callback(clock)));
+                imp.tick_callback.replace(Some(callback_id));
             }
             DecoderUpdate::UnsupportedFormat => {
                 if !self.is_unsupported() {
@@ -856,10 +855,10 @@ impl LpImage {
     /// Roughly called for every frame if image is visible
     ///
     /// We handle advancing to the next frame for animated GIFs etc here.
-    fn tick_callback(&self, clock: &gdk::FrameClock) -> glib::Continue {
+    fn tick_callback(&self, clock: &gdk::FrameClock) -> glib::ControlFlow {
         // Do not animate if not visible
         if !self.is_mapped() {
-            return glib::Continue(true);
+            return glib::ControlFlow::Continue;
         }
 
         let elapsed = clock.frame_time() - self.imp().last_animated_frame.get();
@@ -876,7 +875,7 @@ impl LpImage {
             }
         }
 
-        glib::Continue(true)
+        glib::ControlFlow::Continue
     }
 
     pub fn is_loaded(&self) -> bool {
@@ -956,6 +955,8 @@ impl LpImage {
         }
 
         imp.file_monitor.replace(monitor.ok());
+
+        self.notify("file");
     }
 
     /// File changed on drive
@@ -967,9 +968,19 @@ impl LpImage {
     ) {
         match event {
             gio::FileMonitorEvent::Renamed => {
-                if let Some(file) = file_b {
+                if let Some(file) = file_b.cloned() {
                     log::debug!("Moved to {}", file.uri());
-                    self.set_file(file);
+                    // current file got replaced with a new one
+                    let file_replace = self.file().map_or(false, |x| x.equal(&file));
+                    self.set_file(&file);
+                    if file_replace {
+                        log::debug!("Image got replaced");
+                        let obj = self.clone();
+                        // TODO: error handling is missing
+                        spawn(async move {
+                            obj.load(&file).await;
+                        });
+                    }
                 }
             }
             gio::FileMonitorEvent::ChangesDoneHint => {
@@ -1160,7 +1171,7 @@ impl LpImage {
 
     /// Maximal zoom allowed for this image
     fn max_zoom(&self) -> f64 {
-        if let Some(decoder::ImageFormat::Svg) = self.format() {
+        if self.format().map_or(false, |x| x.is_svg()) {
             let (width, height) = self.original_dimensions();
             // Avoid division by 0
             let long_side = f64::max(1., i32::max(width, height) as f64);
@@ -1243,6 +1254,10 @@ impl LpImage {
     }
 
     fn set_zoom_target(&self, zoom_target: f64) {
+        if self.imp().zoom_target.get() == zoom_target {
+            return;
+        }
+
         log::debug!("Setting zoom target {zoom_target}");
 
         self.imp().zoom_target.set(zoom_target);
@@ -1255,8 +1270,13 @@ impl LpImage {
     fn request_tiles(&self) {
         if let Some(decoder) = self.imp().decoder.borrow().as_ref() {
             if self.zoom_animation().state() != adw::AnimationState::Playing {
+                // Force minimum tile size of 1000x1000 since with smaller
+                // tiles the tiled rendering advantage disappears
+                let x_inset = f32::min(-3., (self.viewport().width() - 1000.) / 2.);
+                let y_inset = f32::min(-3., (self.viewport().height() - 1000.) / 2.);
+
                 decoder.request(crate::decoder::TileRequest {
-                    viewport: self.viewport().inset_r(-3., -3.),
+                    viewport: self.viewport().inset_r(x_inset, y_inset),
                     area: self.preload_area(),
                     zoom: self.imp().zoom_target.get(),
                 });
@@ -1423,7 +1443,8 @@ impl LpImage {
             animation.set_value_to(zoom);
             animation.play();
         } else {
-            self.set_zoom(zoom)
+            self.set_zoom(zoom);
+            self.set_zoom_target(zoom);
         }
     }
 
@@ -1684,12 +1705,12 @@ impl LpImage {
 
     /// Image format
     pub fn format(&self) -> Option<decoder::ImageFormat> {
-        *self.imp().format.borrow()
+        self.imp().format.borrow().clone()
     }
 
     /// Image format displayable name
     pub fn format_name(&self) -> Option<String> {
-        self.imp().format.borrow().map(|x| x.to_string())
+        self.imp().format.borrow().as_ref().map(|x| x.to_string())
     }
 
     /// Returns decoding error if one occured
@@ -1800,6 +1821,7 @@ impl LpImage {
         let default_color = Self::default_background_color();
 
         gio::spawn_blocking(move || {
+            let mut has_transparency = false;
             let mut bytes_iter = bytes.iter();
             // Number of transparent pixels
             let mut completely_transparent = 0;
@@ -1807,12 +1829,23 @@ impl LpImage {
             let mut bad_contrast = 0;
             'img: loop {
                 for _ in 0..texture.width() {
-                    let Some(r) = bytes_iter.next() else { break 'img; };
-                    let Some(g) = bytes_iter.next() else { break 'img; };
-                    let Some(b) = bytes_iter.next() else { break 'img; };
-                    let Some(a) = bytes_iter.next() else { break 'img; };
+                    let Some(r) = bytes_iter.next() else {
+                        break 'img;
+                    };
+                    let Some(g) = bytes_iter.next() else {
+                        break 'img;
+                    };
+                    let Some(b) = bytes_iter.next() else {
+                        break 'img;
+                    };
+                    let Some(a) = bytes_iter.next() else {
+                        break 'img;
+                    };
+
+                    if *a < 255 {
+                        has_transparency = true;
+                    }
 
-                    // 70% transparency
                     if *a == 0 {
                         completely_transparent += 1;
                     } else {
@@ -1837,6 +1870,11 @@ impl LpImage {
                 }
             }
 
+            if !has_transparency {
+                log::debug!("This image does not have transparency");
+                return Some(default_color);
+            }
+
             let n_pixels = texture.width() * texture.height();
 
             let part_bad_contrast = if completely_transparent < n_pixels {
