@@ -1,4 +1,5 @@
 // Copyright (c) 2020-2022 Christopher Davis
+// Copyright (c) 2023 FineFindus
 // Copyright (c) 2023 Sophie Herold
 //
 // This program is free software: you can redistribute it and/or modify
@@ -16,6 +17,7 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+mod drag_overlay;
 mod image;
 mod image_page;
 mod image_view;
@@ -24,6 +26,7 @@ mod print_preview;
 mod properties_view;
 mod sliding_view;
 
+pub use drag_overlay::LpDragOverlay;
 pub use image::LpImage;
 pub use image_page::LpImagePage;
 pub use image_view::LpImageView;
