@@ -1,5 +1,5 @@
-// Copyright (c) 2023 Sophie Herold
 // Copyright (c) 2023 Christopher Davis
+// Copyright (c) 2023 Sophie Herold
 // Copyright (c) 2023 Lubosz Sarnecki
 //
 // This program is free software: you can redistribute it and/or modify
@@ -27,9 +27,9 @@ use crate::widgets::LpImagePage;
 use adw::prelude::*;
 use adw::subclass::prelude::*;
 use indexmap::IndexMap;
-use once_cell::sync::{Lazy, OnceCell};
+use once_cell::sync::Lazy;
 
-use std::cell::{Cell, RefCell};
+use std::cell::{Cell, OnceCell, RefCell};
 
 static SCROLL_DAMPING_RATIO: f64 = 1.;
 static SCROLL_MASS: f64 = 0.5;
@@ -107,6 +107,8 @@ mod imp {
             let swipe_tracker = adw::SwipeTracker::builder()
                 .swipeable(&*self.obj())
                 .reversed(self.is_rtl())
+                .lower_overshoot(true)
+                .upper_overshoot(true)
                 .build();
 
             swipe_tracker.connect_begin_swipe(
@@ -130,22 +132,30 @@ mod imp {
                 gtk::EventControllerScroll::new(gtk::EventControllerScrollFlags::HORIZONTAL);
 
             scroll_controller.connect_scroll(
-                glib::clone!(@weak obj => @default-return gtk::Inhibit(false), move |_, x, _| {
+                glib::clone!(@weak obj => @default-return glib::Propagation::Proceed, move |_, x, _| {
                     let direction_sign = if obj.imp().is_rtl() { -1. } else { 1. };
 
                     if x * direction_sign > 0. {
                         // check end
                         if let Some(max) = obj.imp().snap_points().last() {
-                            gtk::Inhibit(obj.position() >= *max)
+                            if obj.position() >= *max {
+                                glib::Propagation::Stop
+                            } else {
+                                glib::Propagation::Proceed
+                            }
                         } else {
-                            gtk::Inhibit(true)
+                            glib::Propagation::Stop
                         }
                     } else {
                         //check beginning
                         if let Some(min) = obj.imp().snap_points().first() {
-                            gtk::Inhibit(obj.position() <= *min)
+                            if obj.position() <= *min {
+                                glib::Propagation::Stop
+                            } else {
+                                glib::Propagation::Proceed
+                            }
                         } else {
-                            gtk::Inhibit(true)
+                            glib::Propagation::Stop
                         }
                     }
                 }),
@@ -228,6 +238,8 @@ mod imp {
 
         fn distance(&self) -> f64 {
             let obj = self.obj();
+            // TODO: Fix deprecation
+            #[allow(deprecated)]
             let width = obj.allocation().width();
 
             width as f64 + self.page_spacing(width) as f64
